-- Made By Frugle. Refactored by Zivres.

local Frames = {}
local OriginalFrames = {}
OriginalFrames["player"] = PlayerFrame
OriginalFrames["target"] = TargetFrame
OriginalFrames["boss1"] = Boss1TargetFrame
OriginalFrames["boss2"] = Boss2TargetFrame
OriginalFrames["boss3"] = Boss3TargetFrame
OriginalFrames["boss4"] = Boss4TargetFrame

local addonLoadedFrame = CreateFrame("FRAME", "addonLoadedFrame")
addonLoadedFrame:RegisterEvent("ADDON_LOADED")
addonLoadedFrame:SetScript("OnEvent", function (self, event, ...)
	Init(...)
end)

local playerLoggedInFrame = CreateFrame("FRAME", "playerLoggedInFrame")
playerLoggedInFrame:RegisterEvent("PLAYER_LOGIN")
playerLoggedInFrame:SetScript("OnEvent", function (self, event, ...)
	UpdateResources("player")
	UpdateResources("boss1")
	UpdateResources("boss2")
	UpdateResources("boss3")
	UpdateResources("boss4")
end)

local targetChangeFrame = CreateFrame("FRAME", "targetChangeFrame")
targetChangeFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
targetChangeFrame:SetScript("OnEvent", function (self, event, ...)
	UpdateResources("target")
end)

local hpHandlerFrame = CreateFrame("FRAME", "hpHandlerFrame")
hpHandlerFrame:RegisterEvent("UNIT_HEALTH_FREQUENT")
hpHandlerFrame:SetScript("OnEvent", function (self, event, ...)
	UpdateResource(..., "health")
end)

local powerHandlerFrame = CreateFrame("FRAME", "powerHandlerFrame")
powerHandlerFrame:RegisterEvent("UNIT_POWER_UPDATE")
powerHandlerFrame:RegisterEvent("UNIT_DISPLAYPOWER")
powerHandlerFrame:SetScript("OnEvent", function (self, event, ...)
	UpdateResource(..., "mana")
end)

local bossFightStartFrame = CreateFrame("FRAME", "bossFightStartFrame")
bossFightStartFrame:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")
bossFightStartFrame:SetScript("OnEvent", function (self, event, ...)
	UpdateResources("boss1")
	UpdateResources("boss2")
	UpdateResources("boss3")
	UpdateResources("boss4")
end)





function Init(addon)
	if addon == "ANewSimplePercent" then
		if SimplePercent == nil then
			SimplePercent = {}
		end
		InitFrames("player")
		InitFrames("target")
		InitFrames("boss1")
		InitFrames("boss2")
		InitFrames("boss3")
		InitFrames("boss4")
	end
end

function InitFrames(unitId)
	InitFrame(unitId, "health")
	InitFrame(unitId, "mana")
end

function InitFrame(unitId, type)
	local setting = unitId .. type
	if SimplePercent[setting] == nil then
		SimplePercent[setting] = true
	end
	local frame = OriginalFrames[unitId]:CreateFontString(setting .. "Frame", "OVERLAY")
	Frames[setting] = frame
	frame:SetFontObject(PlayerFrameHealthBarText:GetFontObject())
	frame:SetJustifyV("TOP")

	if unitId == "player" then
		frame:SetJustifyH("RIGHT")
	else
		frame:SetJustifyH("LEFT")
	end
	if type == "health" then
		frame.PointYos = 5
	else
		frame.PointYos = -8
	end
	SetFrameState(frame, SimplePercent[setting])
end

function UpdateResources(unitId)
	UpdateResource(unitId, "health")
	UpdateResource(unitId, "mana")
end

function UpdateResource(unitId, type)
	if unitId ~= "player" and unitId ~= "target" and unitId ~= "boss1" and unitId ~= "boss2" and unitId ~= "boss3" and unitId ~= "boss4" then
		return
	end

	local frameId = unitId .. type

	if SimplePercent[frameId] == false then
		return
	end

	local resource = UnitResourse(unitId, type)
	local maxResource = UnitResourseMax(unitId, type)
	
	local percentage
	if resource == 0 or UnitIsDeadOrGhost(unitId) then
		percentage = 0
	elseif resource == maxResource then
		percentage = 100
	elseif maxResource == 100 then
		percentage = resource
	else
		percentage = math.floor(resource / maxResource * 100)
	end

	local frame = Frames[frameId]
	if frame.percentage == percentage then
		return
	end

	frame.percentage = percentage
	local percentageString = tostring(percentage) .. "%"
	frame:SetText(percentageString)

	if unitId == "player" then
		frame.PointXos = 0
		frame:SetPoint("LEFT", OriginalFrames[unitId], "RIGHT", frame.PointXos, frame.PointYos)
	else
		frame.PointXos = frame:GetStringWidth(percentageString) * -1
		frame:SetPoint("LEFT", OriginalFrames[unitId], "LEFT", frame.PointXos, frame.PointYos)
	end
end

function UnitResourse(unitId, type)
	if type == "health" then
		return UnitHealth(unitId)
	end
	return UnitPower(unitId)
end

function UnitResourseMax(unitId, type)
	if type == "health" then
		return UnitHealthMax(unitId)
	end
	return UnitPowerMax(unitId)
end

function SetFrameState(frame, isVisible)
	if isVisible == false then
		frame:Hide()
	else
		frame:Show()
	end
end


SLASH_CHANGE_COMMAND1 = "/simplepercent"
SlashCmdList["CHANGE_COMMAND"] = function (msg, editbox)
	if msg == "player health" then
		HandleCommandAndPrintReaction("player", "health", "Player")
	elseif msg == "player mana" then
		HandleCommandAndPrintReaction("player", "mana", "Player")
	elseif msg == "target health" then
		HandleCommandAndPrintReaction("target", "health", "Target")
	elseif msg == "target mana" then
		HandleCommandAndPrintReaction("target", "mana", "Target")
	elseif msg == "boss health" then
		HandleCommand("boss1", "health")
		HandleCommand("boss2", "health")
		HandleCommand("boss3", "health")
		HandleCommand("boss4", "health")
		PrintReactionMessage("boss1", "health", "Boss additional frame")
	elseif msg == "boss mana" then
		HandleCommand("boss1", "mana")
		HandleCommand("boss2", "mana")
		HandleCommand("boss3", "mana")
		HandleCommand("boss4", "mana")
		PrintReactionMessage("boss1", "mana", "Boss additional frame")
	else
		print("You can toggle percent visiblity with this command")
		print("Syntax: /simplepercent <player or target or boss> <health or mana> | Example: /simplepercent target health")
	end
end

function HandleCommandAndPrintReaction(unitId, type, unitDisplayName)
	HandleCommand(unitId, type)
	PrintReactionMessage(unitId, type, unitDisplayName)
end

function HandleCommand(unitId, type)
	local frameId = unitId .. type

	if SimplePercent[frameId] == false then
		SimplePercent[frameId] = true
	else
		SimplePercent[frameId] = false
	end
	SetFrameState(Frames[frameId], SimplePercent[frameId])
	UpdateResource(unitId, type)
end

function PrintReactionMessage(unitId, type, unitDisplayName)
	local frameId = unitId .. type
	local frame = Frames[frameId]

	if SimplePercent[frameId] == true then
		print(unitDisplayName .. " " .. type .. " percent is now visible.")
	else
		print(unitDisplayName .. " " .. type .. " percent is now hidden.")
	end
end
